from logging import debug
import uvicorn
from fastapi import FastAPI
from api.routes.api import router as api_router

from core.config import DEBUG, PROJECT_NAME, API_PREFIX, VERSION
def get_application() -> FastAPI:
    application = FastAPI(title=PROJECT_NAME, debug=DEBUG, version=VERSION)
    application.include_router(api_router, prefix=API_PREFIX)
    
    return application

app = get_application()

if __name__ == "__main__":
    uvicorn.run("main:app", host="0.0.0.0", port = 8080, reload = False, debug = False)


from fastapi import APIRouter
from fastapi.routing import APIRoute

from api.routes import predictor

router = APIRouter()
router.include_router(predictor.router, tags = ["predictor"], prefix = "/v1")


import logging
import sys
from typing import List

from core.logging import InterceptHandler
from loguru import logger
from starlette.config import Config
from starlette.datastructures import CommaSeparatedStrings, Secret

config = Config(".env")

API_PREFIX = "/api"
VERSION = config("VERSION")
DEBUG : bool = config("DEBUG", cast=bool, default=False)
MAX_CONNECTIONS_COUNT : int = config("MAX_CONNECTIONS_COUNT", cast= int, default=100)
MIN_CONNECTIONS_ACOUNT : int = config("MIN_CONNECTIONS_COUNT", cast=int, default=0)
SECRET_KEY : Secret = config("SECRET_KEY", cast=Secret, default="")

PROJECT_NAME : str = config("PROJECT_NAME")

# logging configuration
LOGGING_LEVEL = logging.DEBUG if DEBUG else logging.INFO
logging.basicConfig(
    handlers=[InterceptHandler(level=LOGGING_LEVEL)], level=LOGGING_LEVEL
)

logger.configure(handlers=[{"sink": sys.stderr, "level": LOGGING_LEVEL}])

MODEL_PATH = config("MODEL_PATH")
MODEL_NAME = config("MODEL_NAME")
MODEL_ID = config("MODEL_ID")


from typing import Callable

from fastapi import FastAPI


def preload_model():
    """
    In order to load model on memory to each worker
    """
    from services.predict import MachineLearningModelHandlerScore

    MachineLearningModelHandlerScore.get_model()


def create_start_app_handler(app: FastAPI) -> Callable:
    def start_app() -> None:
        preload_model()

    return start_app



import os
import sys
from typing import List
from app.core.errors import PredictException, ModelLoadException
from app.core.config import MODEL_NAME, MODEL_PATH, MODEL_ID
from loguru import logger
# sys.path.append(os.path.abspath(os.path.join('..', 'backend-module')))
# import ml.libs.annhub as annhub
from ml.libs import annhub

class MachineLearningModel(object):
    def __init__(self) -> None:
        super().__init__()
        self.load()

    def load(self):
        if MODEL_PATH.endswith("/"):
            path = f"{MODEL_PATH}{MODEL_NAME}"
        else:
            path = f"{MODEL_PATH}/{MODEL_NAME}"

        path = os.path.abspath(os.path.join(os.getcwd(), path))

        if not os.path.exists(path):
            message = f"Machine learning model at {path} not exists!"
            logger.error(message)
            raise FileNotFoundError(message)

        self.load_model(path)
    
    def load_model(self, path : str):
        self.model = annhub.ANNHUB(int(MODEL_ID))
        self.model.LoadWeightFile(path)
        if not self.model:
            message = f"Model {MODEL_ID} could not load!"
            logger.error(message)
            raise ModelLoadException(message)

    def predict(self, data_input : List[float]) -> List[float]:
        self.model.Predict(data_input)
        output = self.model.GetOutput()
        if not output:
            raise PredictException(f"Error when predict input:'{data_input}'")
        return output





import logging

from loguru import logger


class InterceptHandler(logging.Handler):
    def emit(self, record: logging.LogRecord) -> None:  # pragma: no cover
        logger_opt = logger.opt(depth=7, exception=record.exc_info)
        logger_opt.log(record.levelname, record.getMessage())
